using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Networking;

namespace Nebula
{
    public class FYHttpService : MonoBehaviour
    {
        public static int HttpServiceStatus_Success = 0;
        public static int HttpServiceStatus_RequestError = -1;
        public static int HttpServiceStatus_ParamError = -2;

        public static string HttpServiceHint_Success = "成功";
        public static string HttpServiceHint_ParamError = "响应参数解析失败";


        private static string PRO_URL = "https://gate-nebula.737.com";
        private static string SIT_URL = "https://sit-gate-nebula.737.com";
        private static string DEV_URL = "http://dev-gate-nebula.qyy.com";
        

        private string BaseUrl
        {
            get
            {
                if (NebulaSDKBridge.Instance.ConfigModel.env == "sit")
                {
                    return SIT_URL;
                }
                else if (NebulaSDKBridge.Instance.ConfigModel.env == "dev")
                {
                    return DEV_URL;
                }
                else
                {
                    return PRO_URL;
                }
            }
        }

        private static FYHttpService instance;
        public static FYHttpService Instance
        {
            get
            {
                if (instance == null)
                {
                    GameObject go = new GameObject("FYHttpService");
                    instance = go.AddComponent<FYHttpService>();
                }
                return instance;
            }
        }

        private void Awake()
        {
        }

        public void UserDetail(string user_id, string code, Action<int, string, FYResponseModel.BaseResponse<FYResponseModel.UserDetail>> action)
        {
            FYRequestModel.UserDetail model = new FYRequestModel.UserDetail();
            model.company_id = NebulaSDKBridge.Instance.ConfigModel.nbCompanyId;
            model.app_id = NebulaSDKBridge.Instance.ConfigModel.nbAppId;
            model.channel_id = NebulaSDKBridge.Instance.ConfigModel.nbChannelId;
            model.union_id = "";
            model.open_id = "";
            model.device_id = NebulaSDKBridge.Instance.DeviceId;
            model.startup_id = NebulaSDKBridge.Instance.StartupId;
            model.payment_code = NebulaSDKBridge.Instance.ConfigModel.paymentCode;
            model.passport_code = NebulaSDKBridge.Instance.ConfigModel.passportCode;
            string os = TTSDK.TT.GetSystemInfo().platform;
            if (os == "android")
            {
                model.os = 1;
            }
            else if (os == "ios")
            {
                model.os = 0;
            }
            else
            {
                model.os = 2;
            }
            model.network_mode = "0";
            model.device_type = TTSDK.TT.GetSystemInfo().model;
            model.os_version = TTSDK.TT.TTSDKVersion;
            model.sdk_version = NebulaSDKBridge.SDKVersion;
            model.game_version = NebulaSDKBridge.Instance.GameVersion;
            model.timestamp = $"{System.DateTimeOffset.UtcNow.ToUnixTimeSeconds()}";
            model.is_debug = NebulaSDKBridge.Instance.IsDebug ? 1 : 0;
            model.data_version = "1";
            model.sign_type = "md5";
            model.sign_nonce = FYHelper.RandomString(8);
            model.sign_version = "1.0";

            model.user_id = user_id;
            model.code = code;

            string sign = FYHelper.GetSign(model, NebulaSDKBridge.Instance.ConfigModel.nbAppKey);
            model.sign = sign;


            FYHttpHelper.Instance.Post($"{BaseUrl}/account/user/detail", model, (bool isSuccess, string result) =>
            {
                if (isSuccess)
                {
                    var info = JsonUtility.FromJson<FYResponseModel.BaseResponse<FYResponseModel.UserDetail>>(result);

                    if (info != null)
                    {
                        action?.Invoke(HttpServiceStatus_Success, HttpServiceHint_Success, info);
                    }
                    else
                    {
                        action?.Invoke(HttpServiceStatus_ParamError, HttpServiceHint_ParamError, null);
                    }

                }
                else
                {
                    action?.Invoke(HttpServiceStatus_RequestError, result, null);
                }
            });
        }

        public void OrderCreate(string pay_amount, string game_order_id, string goods_id, string goods_name,
            string player_id, string player_name, string player_level, string vip_level, string server_id,
            string user_id, string open_id, string union_id, string notify_ext, string notify_url,
            Action<int, string, FYResponseModel.BaseResponse<FYResponseModel.OrderCreate>> action)
        {
            FYRequestModel.OrderCreate model = new FYRequestModel.OrderCreate();
            model.company_id = NebulaSDKBridge.Instance.ConfigModel.nbCompanyId;
            model.app_id = NebulaSDKBridge.Instance.ConfigModel.nbAppId;
            model.channel_id = NebulaSDKBridge.Instance.ConfigModel.nbChannelId;
            model.union_id = union_id;
            model.open_id = open_id;
            model.device_id = NebulaSDKBridge.Instance.DeviceId;
            model.startup_id = NebulaSDKBridge.Instance.StartupId;
            model.payment_code = NebulaSDKBridge.Instance.ConfigModel.paymentCode;
            model.passport_code = NebulaSDKBridge.Instance.ConfigModel.passportCode;
            string os = TTSDK.TT.GetSystemInfo().platform;
            if (os == "android")
            {
                model.os = 1;
            }
            else if (os == "ios")
            {
                model.os = 0;
            }
            else
            {
                model.os = 2;
            }
            model.network_mode = "0";
            model.device_type = TTSDK.TT.GetSystemInfo().model;
            model.os_version = TTSDK.TT.GetSystemInfo().platform;
            model.sdk_version = $"{NebulaSDKBridge.SDKVersion}/{TTSDK.TT.TTSDKVersion}";
            model.game_version = NebulaSDKBridge.Instance.GameVersion;
            model.timestamp = $"{System.DateTimeOffset.UtcNow.ToUnixTimeSeconds()}";
            model.is_debug = NebulaSDKBridge.Instance.IsDebug ? 1 : 0;
            model.data_version = "1";
            model.sign_type = "md5";
            model.sign_nonce = FYHelper.RandomString(8);
            model.sign_version = "1.0";

            model.pay_amount = pay_amount;
            model.game_order_id = game_order_id;
            model.goods_id = goods_id;
            model.goods_name = goods_name;
            model.player_id = player_id;
            model.role_id = FYHelper.GetMD5($"{server_id}{player_id}");
            model.player_name = player_name;
            model.player_level = player_level;
            model.vip_level = vip_level;
            model.server_id = server_id;
            model.user_id = user_id;
            model.pay_sign_type = "md5";
            model.notify_url = notify_url;
            model.notify_ext = notify_ext;
            model.payment_channel = 0;
            model.channel_ext = "";

            string sign = FYHelper.GetSign(model, NebulaSDKBridge.Instance.ConfigModel.nbAppKey);
            model.sign = sign;


            FYHttpHelper.Instance.Post($"{BaseUrl}/pay/order/create", model, (bool isSuccess, string result) =>
            {
                if (isSuccess)
                {
                    var info = JsonUtility.FromJson<FYResponseModel.BaseResponse<FYResponseModel.OrderCreate>>(result);

                    if (info != null)
                    {
                        action?.Invoke(HttpServiceStatus_Success, HttpServiceHint_Success, info);
                    }
                    else
                    {
                        action?.Invoke(HttpServiceStatus_ParamError, HttpServiceHint_ParamError, null);
                    }

                }
                else
                {
                    action?.Invoke(HttpServiceStatus_RequestError, result, null);
                }
            });
        }


        public void PayConfig(Action<int, string, FYResponseModel.BaseResponse<FYResponseModel.PayConfig>> action)
        {
            FYRequestModel.BaseParams model = new FYRequestModel.BaseParams();
            model.company_id = NebulaSDKBridge.Instance.ConfigModel.nbCompanyId;
            model.app_id = NebulaSDKBridge.Instance.ConfigModel.nbAppId;
            model.channel_id = NebulaSDKBridge.Instance.ConfigModel.nbChannelId;
            model.union_id = "";
            model.open_id = "";
            model.device_id = NebulaSDKBridge.Instance.DeviceId;
            model.startup_id = NebulaSDKBridge.Instance.StartupId;
            model.payment_code = NebulaSDKBridge.Instance.ConfigModel.paymentCode;
            model.passport_code = NebulaSDKBridge.Instance.ConfigModel.passportCode;
            string os = TTSDK.TT.GetSystemInfo().platform;
            if (os == "android")
            {
                model.os = 1;
            }
            else if (os == "ios")
            {
                model.os = 0;
            }
            else
            {
                model.os = 2;
            }
            model.network_mode = "0";
            model.device_type = TTSDK.TT.GetSystemInfo().model;
            model.os_version = TTSDK.TT.TTSDKVersion;
            model.sdk_version = NebulaSDKBridge.SDKVersion;
            model.game_version = NebulaSDKBridge.Instance.GameVersion;
            model.timestamp = $"{System.DateTimeOffset.UtcNow.ToUnixTimeSeconds()}";
            model.is_debug = NebulaSDKBridge.Instance.IsDebug ? 1 : 0;
            model.data_version = "1";
            model.sign_type = "md5";
            model.sign_nonce = FYHelper.RandomString(8);
            model.sign_version = "1.0";

            string sign = FYHelper.GetSign(model, NebulaSDKBridge.Instance.ConfigModel.nbAppKey);
            model.sign = sign;

            Dictionary<string, string> dict = FYHelper.ModelToDic<FYRequestModel.BaseParams>(model);

            FYHttpHelper.Instance.Get($"{BaseUrl}/pay/channelConfig/get", dict, (bool isSuccess, string result) =>
            {
                if (isSuccess)
                {
                    var info = JsonUtility.FromJson<FYResponseModel.BaseResponse<FYResponseModel.PayConfig>>(result);

                    if (info != null)
                    {
                        action?.Invoke(HttpServiceStatus_Success, HttpServiceHint_Success, info);
                    }
                    else
                    {
                        action?.Invoke(HttpServiceStatus_ParamError, HttpServiceHint_ParamError, null);
                    }

                }
                else
                {
                    action?.Invoke(HttpServiceStatus_RequestError, result, null);
                }
            });
        }

    }
}