using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Text;
using TTSDK.UNBridgeLib.LitJson​;

namespace Nebula
{
    public class FYLogService
    {
        public static string EXCEPTION = "3001";
        public static string NETWORK = "2001";
        public static string INIT = "16003";
        public static string LOGIN = "16004";
        public static string PAY = "16005";
        public static string REWOARD_AD = "16006";
        public static string CUSTOMER_SERVICE = "16007";
        public static string THIRD = "16008";
        public static string CALL = "16009";

        public static int seq = 0;

        private string eventId = "";
        private string desc = "";
        private string project = "";
        private string projectVersion = "";
        private string level = "info";
        private string type = "defualt";
        private Dictionary<string, string> extraDict = new Dictionary<string, string>();
        private Dictionary<string, string> baseDict = new Dictionary<string, string>();

        public static FYLogService Create()
        {
            return new FYLogService();
        }

        // 链式调用方法
        public FYLogService SetEventId(string eventId)
        {
            this.eventId = eventId;
            return this;
        }

        public FYLogService SetDesc(string desc)
        {
            this.desc = desc;
            return this;
        }

        public FYLogService AddExtra(string key, string value)
        {
            this.extraDict.Add(key, value);
            return this;
        }

        public FYLogService AddBase(string key, string value)
        {
            this.baseDict.Add(key, value);
            return this;
        }

        public FYLogService SetProject(string project)
        {
            this.project = project;
            return this;
        }

        public FYLogService SetProjectVersion(string projectVersion)
        {
            this.projectVersion = projectVersion;
            return this;
        }

        public FYLogService SetLevel(string level)
        {
            this.level = level;
            return this;
        }

        public FYLogService SetType(string type)
        {
            this.type = type;
            return this;
        }

        public void Report()
        {
            string project = "";
            if (NebulaSDKBridge.Instance.ConfigModel.env == "sit")
            {
                project = "sit-feishu-develop-tracker";
            }
            else if (NebulaSDKBridge.Instance.ConfigModel.env == "dev")
            {
                project = "sit-feishu-develop-tracker";
            }
            else
            {
                project = "feishu-develop-tracker";
            }

            AddBase("appId", NebulaSDKBridge.Instance.ConfigModel.nbAppId);
            AddBase("channelId", NebulaSDKBridge.Instance.ConfigModel.nbChannelId);
            AddBase("channelTemplateCode", NebulaSDKBridge.Instance.ConfigModel.nbChannelTemplateCode);
            AddBase("passportCode", NebulaSDKBridge.Instance.ConfigModel.passportCode);
            AddBase("paymentCode", NebulaSDKBridge.Instance.ConfigModel.paymentCode);

            string os = TTSDK.TT.GetSystemInfo().platform;
            string osVersion = TTSDK.TT.GetSystemInfo().system;
            string deviceName = TTSDK.TT.GetSystemInfo().model;

            seq = seq + 1;

            var paramsDict = new Dictionary<string, string>
            {
                {"event_id", this.eventId},
                {"desc", this.desc},
                {"extra", CreateParams(this.extraDict)},
                {"base", CreateParams(this.baseDict)},
                {"project", NebulaSDKBridge.SDKName},
                {"project_version", NebulaSDKBridge.SDKVersion},
                {"level", this.level},
                {"os", os},
                {"os_version", osVersion },
                {"startup_id", NebulaSDKBridge.Instance.StartupId },
                {"time", FYHelper.GetCurrentTimeMilliseconds() },
                {"device_id", NebulaSDKBridge.Instance.DeviceId},
                {"device_name", deviceName },
                {"timestamp", $"{FYHelper.GetTimestampMilliseconds()}" },
                {"type", this.type },
                {"seq", $"{seq}"},
                {"APIVersion", "0.6.0"}
            };


            string logstore = "mobile_track";
            string serviceAddr = "cn-shanghai.log.aliyuncs.com";

            string url = $"https://{project}.{serviceAddr}/logstores/{logstore}/track";

            FYHttpHelper.Instance.Get(url, paramsDict, (bool isSuccess, string result) =>
            {
                if (isSuccess)
                {

                }
                else
                {
                    
                }
            });
        }

        private string CreateParams(Dictionary<string, string> paramsDict)
        {

            var jsonData = new JsonData();

            foreach (var kvp in paramsDict)
            {
                JsonData valueJsonData;
                try
                {
                    valueJsonData = JsonMapper.ToObject(kvp.Value);
                    jsonData[kvp.Key] = valueJsonData;
                }
                catch (JsonException e)
                {
                    jsonData[kvp.Key] = kvp.Value;
                }
                
            }

            return jsonData.ToJson();
        }
    }
}
    
