using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TTSDK;
using System;
using TTSDK.UNBridgeLib.LitJson​;

namespace Nebula
{
    public class NebulaSDKBridge
    {
        public static string SDKName = "NebulaDouyin";

        /// <summary>
        /// 10348
        /// 2024.09.14
        /// 新增客服及侧边栏集成
        /// iOS支付变更为发起抖音钻石支付
        ///
        /// 3.3.66
        /// 2024.09.14
        /// 对外发布
        ///
        /// 10358
        /// 2024.09.30
        /// 新增集成创建快捷方式
        ///
        /// 3.3.69
        /// 2024.10.09
        /// 对外发布
        ///
        /// 10360
        /// 2024.10.15
        /// 修复Android端金额问题
        /// 
        /// 3.3.70
        /// 2024.10.15
        /// 对外发布
        ///
        /// 10361
        /// 2024.10.21
        /// 解决今日头条无法拉起客服的问题
        /// 
        /// 3.3.71
        /// 2024.10.21
        /// 对外发布
        ///
        /// 10362
        /// 2024.10.22
        /// 解决特殊符号造成的签名不一致问题
        ///
        /// 3.3.72
        /// 2024.10.22
        /// 对外发布
        ///
        /// 10370
        /// 2024.11.18
        /// 抖小IM客服支付功能集成
        ///
        /// 3.3.75
        /// 2024.11.18
        /// 对外发布
        ///
        /// 10375
        /// 2024.12.10
        /// 网络请求工具优化
        /// 抖音SDK升级
        /// 添加日志上报
        /// 推荐流直出功能
        ///
        /// 3.3.80
        /// 2024.12.17
        /// 对外发布
        /// </summary>
        public static string SDKVersion = "3.3.80";
        public static string SDKInnerVersion = "10375";

        private static NebulaSDKBridge instance;
        public static NebulaSDKBridge Instance
        {
            get
            {
                if (null == instance)
                {
                    instance = new NebulaSDKBridge();
                    GameObject go = new GameObject("NebulaSDKBridgeCallback");
                    GameObject.DontDestroyOnLoad(go);
                    go.AddComponent<NebulaSDKBridgeCallback>();
                }
                return instance;
            }
        }

        private NebulaSDKBridge()
        {
            Debug.Log("###Application.logMessageReceived");
            Application.logMessageReceived += HandleLog;
        }

        /// <summary>
        /// 监听异常日志
        /// </summary>
        /// <param name="logString"></param>
        /// <param name="stackTrace"></param>
        /// <param name="type"></param>
        private void HandleLog(string logString, string stackTrace, LogType type)
        {
            if (type == LogType.Exception || type == LogType.Error)
            {
                FYLogService.Create().SetEventId(FYLogService.EXCEPTION).SetDesc($"发生异常-{logString}").AddExtra("stackTrace", stackTrace).SetLevel($"{type}").Report();
            }
        }

        private FYConfigModel configModel;
        public FYConfigModel ConfigModel
        {
            get
            {
                return configModel;
            }
        }

        private bool isDebug;
        public bool IsDebug
        {
            get
            {
                var type = TTSDK.TT.s_ContainerEnv.GetVersionType();
                if (type == VersionType.Release)
                {
                    isDebug = false;
                }
                return isDebug;
            }
        }

        private string gameVersion;
        public string GameVersion
        {
            get
            {
                if (null == gameVersion)
                {
                    return "";
                }
                return gameVersion;
            }
        }

        private string deviceId;
        public string DeviceId
        {
            get
            {
                if (deviceId == null)
                {
                    deviceId = FYHelper.DeviceId();
                }
                return deviceId;
            }
        }

        private string startupId;
        public string StartupId
        {
            get
            {
                if (startupId == null)
                {
                    startupId = FYHelper.StartupId();
                }
                return startupId;
            }
        }

        /// <summary>
        /// 登录数据
        /// </summary>
        private NebulaSDKLoginData loginData;
        public NebulaSDKLoginData LoginData
        {
            get
            {
                return loginData;
            }
        }

        /// <summary>
        /// 是否支持侧边栏
        /// </summary>
        private bool supportSideBar;


        public void Init(string gameVersion)
        {
            Debug.Log($"###NebulaSDKBridge Init>>gameVersion:{gameVersion}");

            this.gameVersion = gameVersion;

            var json = Resources.Load<TextAsset>("fy-config");
            if (json == null)
            {
                Debug.LogError("###配置文件fy-config.json无法读取");
                
                var info0 = new NebulaSDKInitInfo();
                info0.status = "error";
                info0.code = NebulaSDKCode.Code10009;
                info0.msg = NebulaSDKCode.Msg10009;

                FYLogService.Create().SetEventId(FYLogService.INIT).SetDesc("初始化业务-配置文件无法读取-回调NebulaSDKBridgeCallback.InitCallback").Report();

                NebulaSDKBridgeCallback.InitCallback?.Invoke(info0);
                return;
            }

            this.configModel = JsonUtility.FromJson<FYConfigModel>(json.text);

            FYLogService.Create().SetEventId(FYLogService.INIT).SetDesc("初始化业务-NebulaSDKBridge.Init").AddExtra("gameVersion", gameVersion).Report();

            TT.InitSDK((code, env) =>
            {

                if (code == 0)
                {
                    var info = new NebulaSDKInitInfo();
                    info.status = "success";
                    info.code = 0;
                    info.msg = "初始化成功";
                    var data = new NebulaSDKInitData();
                    data.nbAppId = this.configModel.nbAppId;
                    data.nbChannelId = this.configModel.nbChannelId;
                    info.data = data;

                    FYLogService.Create().SetEventId(FYLogService.INIT).SetDesc("初始化业务-回调NebulaSDKBridgeCallback.InitCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                    NebulaSDKBridgeCallback.InitCallback?.Invoke(info);
                }
                else
                {
                    var info = new NebulaSDKInitInfo();
                    info.status = "error";
                    info.code = 0;
                    info.msg = "初始化失败";
                    var data = new NebulaSDKInitData();
                    data.nbAppId = this.configModel.nbAppId;
                    data.nbChannelId = this.configModel.nbChannelId;
                    info.data = data;

                    FYLogService.Create().SetEventId(FYLogService.INIT).SetDesc("初始化业务-回调NebulaSDKBridgeCallback.InitCallback").AddExtra("info", JsonUtility.ToJson(info)).AddExtra("code", $"{code}").Report();

                    NebulaSDKBridgeCallback.InitCallback?.Invoke(info);
                }

                // 设置监听抖音OnShow
                TT.GetAppLifeCycle().OnShow += OnShow;
                // 获取启动参数
                GetLaunchOptions();
            });
        }

        
        public void Login()
        {
            Debug.Log($"###NebulaSDKBridge Login");

            FYLogService.Create().SetEventId(FYLogService.LOGIN).SetDesc("登录业务-NebulaSDKBridge.Login").Report();

            TT.Login((code, anonymousCode, isLogin) =>
            {
                Debug.Log($"###Douyin Login Success>>code:{code}>>anonymousCode:{anonymousCode}>>isLogin:{isLogin}");

                FYHttpService.Instance.UserDetail("0", code, (int status, string message, FYResponseModel.BaseResponse<FYResponseModel.UserDetail> userDetail) =>
                {
                    if (status == FYHttpService.HttpServiceStatus_Success)
                    {
                        if (userDetail.status == 0)
                        {
                            var info = new NebulaSDKLoginInfo();
                            info.status = "success";
                            info.code = 0;
                            info.msg = "登录成功";
                            var data = new NebulaSDKLoginData();
                            data.openId = userDetail.data.user.user_id;
                            data.unionId = userDetail.data.user.u_id;
                            data.token = userDetail.data.token;
                            data.nbOpenId = userDetail.data.open_id;
                            data.nbOpendId = userDetail.data.open_id;
                            data.nbUnionId = userDetail.data.union_id;
                            info.data = data;

                            this.loginData = data;

                            FYLogService.Create().SetEventId(FYLogService.LOGIN).SetDesc("登录业务-登录成功-回调NebulaSDKBridgeCallback.LoginCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                            NebulaSDKBridgeCallback.LoginCallback?.Invoke(info);
                        }
                        else
                        {
                            var info = new NebulaSDKLoginInfo();
                            info.status = "error";
                            info.code = NebulaSDKCode.Code20001;
                            info.msg = userDetail.message;

                            FYLogService.Create().SetEventId(FYLogService.LOGIN).SetDesc("登录业务-登录失败-服务端登录验证失败-回调NebulaSDKBridgeCallback.LoginCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                            NebulaSDKBridgeCallback.LoginCallback?.Invoke(info);
                        }

                    }
                    else
                    {
                        var info = new NebulaSDKLoginInfo();
                        info.status = "error";
                        info.code = NebulaSDKCode.Code20001;
                        info.msg = message;

                        FYLogService.Create().SetEventId(FYLogService.LOGIN).SetDesc("登录业务-登录失败-请求登录验证失败-回调NebulaSDKBridgeCallback.LoginCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                        NebulaSDKBridgeCallback.LoginCallback?.Invoke(info);
                    }
                });

            }, (msg) => {
                Debug.Log($"###Douyin Login Failure>>msg:{msg}");

                var info = new NebulaSDKLoginInfo();
                info.status = "error";
                info.code = NebulaSDKCode.Code20001;
                info.msg = $"登录失败:{msg}";

                FYLogService.Create().SetEventId(FYLogService.LOGIN).SetDesc("登录业务-登录失败-第三方渠道登录失败-回调NebulaSDKBridgeCallback.LoginCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                NebulaSDKBridgeCallback.LoginCallback?.Invoke(info);

            }, true);
        }

        /// <summary>
        /// 支付
        /// </summary>
        /// <param name="payParams">支付参数</param>
        public void Pay(NebulaSDKPayParams payParams)
        {
            FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-NebulaSDKBridge.Pay").AddExtra("payParams", JsonUtility.ToJson(payParams)).Report();
            Pay(payParams.goodsPrice, payParams.tradeNo, payParams.goodsId, payParams.goodsName, payParams.playerId, payParams.playerName, payParams.playerLevel, payParams.vipLevel, payParams.serverId, payParams.zoneId, payParams.sandbox, payParams.notifyURL, payParams.notifyExt);
        }

        /// <summary>
        /// 支付
        /// </summary>
        /// <param name="goodsPrice">商品价格，单位：分</param>
        /// <param name="tradeNo">游戏订单号</param>
        /// <param name="goodsId">商品 ID</param>
        /// <param name="goodsName">商品名称</param>
        /// <param name="playerId">角色 ID</param>
        /// <param name="playerName">玩家名称</param>
        /// <param name="playerLevel">玩家等级</param>
        /// <param name="vipLevel">VIP 等级</param>
        /// <param name="serverId">服务器 ID</param>
        /// <param name="zoneId">分区 ID</param>
        /// <param name="sandbox">沙箱支付，0=正式支付、1=沙箱支付</param>
        /// <param name="notifyURL">回调地址</param>
        /// <param name="notifyExt">额外参数</param>
        public void Pay(int goodsPrice, string tradeNo, string goodsId, string goodsName, string playerId, string playerName, string playerLevel, string vipLevel="0", string serverId="1", string zoneId="1", int sandbox=0, string notifyURL="", string notifyExt="")
        {
            Debug.Log($"###NebulaSDKBridge Pay>>goodsPrice:{goodsPrice},tradeNo:{tradeNo},goodsId:{goodsId},goodsName:{goodsName},playerId:{playerId},playerName:{playerName},playerLevel:{playerLevel},vipLevel:{vipLevel},serverId:{serverId},zoneId:{zoneId},sandbox:{sandbox},notifyURL:{notifyURL},notifyExt:{notifyExt}");

            string os = TTSDK.TT.GetSystemInfo().platform;

            FYHttpService.Instance.PayConfig((int status, string message, FYResponseModel.BaseResponse<FYResponseModel.PayConfig> payConfig) => {
                
                if (os == "android")
                {
                    if (String.IsNullOrEmpty(payConfig.data.param.android_pay_mode) || payConfig.data.param.android_pay_mode == "0")
                    {
                        Debug.Log($"###android暂不支持支付");
                        var info = new NebulaSDKPayInfo();
                        info.status = "error";
                        info.code = NebulaSDKCode.Code30001;
                        info.msg = payConfig.data.param.android_close_pay_guide;

                        FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-支付失败回调-Android暂不支持支付-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).AddExtra("payConfig", JsonUtility.ToJson(payConfig)).Report();

                        NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
                        return;
                    }
                }

                if (os == "ios")
                {
                    if (String.IsNullOrEmpty(payConfig.data.param.ios_pay_mode) || payConfig.data.param.ios_pay_mode == "0")
                    {
                        Debug.Log($"###ios暂不支持支付");
                        var info = new NebulaSDKPayInfo();
                        info.status = "error";
                        info.code = NebulaSDKCode.Code30001;
                        info.msg = payConfig.data.param.ios_close_pay_guide;

                        FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-支付失败回调-iOS暂不支持支付-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).AddExtra("payConfig", JsonUtility.ToJson(payConfig)).Report();

                        NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
                        return;
                    }
                }


                FYHttpService.Instance.OrderCreate($"{goodsPrice}", tradeNo, goodsId, goodsName,
                playerId, playerName, playerLevel, vipLevel, serverId,
                this.loginData.openId, this.loginData.nbOpendId,
                this.loginData.nbUnionId, notifyExt, notifyURL,
                (int status, string message, FYResponseModel.BaseResponse<FYResponseModel.OrderCreate> orderCreate) =>
                {
                    if (status == FYHttpService.HttpServiceStatus_Success)
                    {
                        if (orderCreate.status == 0)
                        {
                            if (os == "android")
                            {
                                DirectPay(zoneId, goodsPrice, orderCreate);
                            }
                            else if (os == "ios")
                            {
                                if (payConfig.data.param.ios_pay_mode == "1")
                                {
                                    DiamondPay(zoneId, goodsPrice, orderCreate);
                                }
                                else if (payConfig.data.param.ios_pay_mode == "2")
                                {
                                    ServicePay(zoneId, goodsPrice, orderCreate);
                                }
                                else
                                {
                                    Debug.Log($"###该支付方式不支持>>ios_pay_mode:{payConfig.data.param.ios_pay_mode}");
                                    var info = new NebulaSDKPayInfo();
                                    info.status = "error";
                                    info.code = NebulaSDKCode.Code30001;
                                    info.msg = $"该支付方式不支持ios_pay_mode:{payConfig.data.param.ios_pay_mode}";

                                    FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-支付失败回调-该支付方式不支持-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).AddExtra("payConfig", JsonUtility.ToJson(payConfig)).Report();

                                    NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
                                }
                            }
                            else
                            {
                                Debug.Log($"###该os不支持支付>>os:{os}");
                                var info = new NebulaSDKPayInfo();
                                info.status = "error";
                                info.code = NebulaSDKCode.Code30001;
                                info.msg = $"该os:{os}不支持支付";

                                FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-支付失败回调-该os不支持支付-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).AddExtra("os", os).Report();

                                NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
                            }
                        }
                        else
                        {
                            var info = new NebulaSDKPayInfo();
                            info.status = "error";
                            info.code = NebulaSDKCode.Code30001;
                            info.msg = orderCreate.message;

                            FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-支付失败回调-服务端创建订单失败-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                            NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
                        }

                    }
                    else
                    {
                        var info = new NebulaSDKPayInfo();
                        info.status = "error";
                        info.code = NebulaSDKCode.Code30001;
                        info.msg = message;

                        FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-支付失败回调-请求创建订单失败-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                        NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
                    }
                });

            });

        }

        /// <summary>
        /// 打开侧边栏
        /// 抖音侧边栏技术指南：https://developer.open-douyin.com/docs/resource/zh-CN/mini-game/develop/guide/open-ability/sidebar/Introduction-for-tech
        /// 抖音侧边栏介绍：https://developer.open-douyin.com/docs/resource/zh-CN/mini-game/operation1/ability/-retention/sidebar
        /// 抖音侧边栏SDK文档：https://developer.open-douyin.com/docs/resource/zh-CN/mini-game/develop/guide/game-engine/rd-to-SCgame/unity-game-access/sc_stark_sdk#215b9294
        /// </summary>
        /// <param name="success">打开成功回调</param>
        /// <param name="fail">打开失败回调</param>
        public void NavigateToSideBar(Action success = null, Action<int, string> fail = null)
        {
            var data = new JsonData
            {
                ["scene"] = "sidebar"
            };

            FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("打开侧边栏业务-NebulaSDKBridge.NavigateToSideBar").Report();

            TT.NavigateToScene(data, () =>
            {
                Debug.Log("###NavigateToSideBar success");
                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("打开侧边栏业务-打开成功").Report();
                if (success != null)
                {
                    success();
                }
            }, () =>
            {
                Debug.Log("###NavigateToSideBar complete");
            }, (errCode, errMsg) =>
            {
                Debug.Log($"###NavigateToSideBar error>>errCode:{errCode}>>errMsg:{errMsg}");
                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("打开侧边栏业务-打开失败").AddExtra("errCode", $"{errCode}").AddExtra("errMsg", errMsg).Report();
                if (fail != null)
                {
                    fail(errCode, errMsg);
                }
            });
        }

        /// <summary>
        /// 打开客服页​
        /// 小6客服介绍文档：https://developer.open-douyin.com/docs/resource/zh-CN/mini-game/operation1/ability/user-service/customer-service-introduce
        /// 抖音IM客服介绍文档：https://developer.open-douyin.com/docs/resource/zh-CN/mini-game/operation1/ability/user-service/douyinIM
        /// 抖音客服SDK文档：https://developer.open-douyin.com/docs/resource/zh-CN/mini-game/develop/guide/game-engine/rd-to-SCgame/unity-game-access/sc_stark_sdk#d40c332e
        /// </summary>
        /// <param name="type">1:小6客服；2:抖音IM 客服(仅支持抖音)</param>
        /// <param name="action">打开回调：true-打开成功；false-打开失败</param>
        public void OpenCustomerServiceConversation(int type, Action<bool> action = null)
        {
            Debug.Log($"###NebulaSDKBridge OpenCustomerServiceConversation>>type:{type}");
            FYLogService.Create().SetEventId(FYLogService.CUSTOMER_SERVICE).SetDesc("打开打开客服业务-NebulaSDKBridge.OpenCustomerServiceConversation").AddExtra("type", $"{type}").Report();
            if (type == 1)
            {
                TT.OpenCustomerServicePage((flag) =>
                {
                    Debug.Log($"###拉起小6客服>>{flag}");
                    FYLogService.Create().SetEventId(FYLogService.CUSTOMER_SERVICE).SetDesc("打开打开客服业务-拉起小6客服").AddExtra("flag", $"{flag}").Report();
                    if (action != null)
                    {
                        action(flag);
                    }
                });
            }
            else
            {
                JsonData data = new JsonData
                {
                    ["type"] = type
                };
                // 仅支持抖音
                // data数据：
                //  type:(1：小6客服 2:抖音IM客服)
                //  sessionFrom:(保留字段，暂时可以不填)
                
                TT.OpenCustomerServiceConversation(data, (succ) => {
                    Debug.Log($"###抖音IM客服>>{succ}");
                    FYLogService.Create().SetEventId(FYLogService.CUSTOMER_SERVICE).SetDesc("打开打开客服业务-拉起抖音IM客服").AddExtra("succ", $"{succ}").Report();
                    if (action != null)
                    {
                        action(succ);
                    }
                });
            }

        }

        /// <summary>
        /// 创建桌面快捷方式​
        /// </summary>
        /// <param name="action">创建回调：true-成功；false-失败</param>
        public void CreateShortcut(Action<bool> action = null)
        {
            FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("创建桌面快捷方式​业务-NebulaSDKBridge.CreateShortcut").Report();
            TT.AddShortcut((succ) =>
            {
                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("创建桌面快捷方式​业务-创建结果").AddExtra("succ", $"{succ}").Report();
                if (action != null)
                {
                    action(succ);
                }
            });
        }

        /// <summary>
        /// 直玩订阅
        /// 向用户请求授权，允许游戏在满足一定的条件后出现在 Feed 流中
        /// </summary>
        /// <param name="scene">订阅的场景ID。1：离线收益场景​；2：体力恢复场景​；3：重要事件掉落​</param>
        /// <param name="contentIDs">自定义文案ID</param>
        /// <param name="action">isSuccess：用户的订阅结果（true为成功，false为订阅失败）；message：回调信息 ​ ​</param>
        public void RequestFeedSubscribe(int scene, string[] contentIDs, Action<bool, string> action = null)
        {
            var tempContentIDs = JsonData.NewJsonArray();

            for (int i = 0; i < contentIDs.Length; i++)
            {
                tempContentIDs.Add(contentIDs[i]);
            }

            var param = new JsonData();
            param["type"] = "play";// play=直玩（该场景必传scene和contentIDs）
            param["scene"] = scene;
            param["contentIDs"] = tempContentIDs;

            FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("直玩订阅业务-NebulaSDKBridge.RequestFeedSubscribe").AddExtra("param", param.ToJson()).Report();

            TT.RequestFeedSubscribe(param, (res) => {
                bool success = (bool)res["success"];
                string errMsg = (string)res["errMsg"];

                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("直玩订阅业务-完成回调").AddExtra("success", $"{success}").AddExtra("errMsg", errMsg).Report();

                action?.Invoke(success, errMsg);
            }, (errNo, errMsg) => {
                bool success = false;

                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("直玩订阅业务-错误回调").AddExtra("success", $"{success}").AddExtra("errNo", $"{errNo}").AddExtra("errMsg", errMsg).Report();

                action?.Invoke(success, errMsg);
            }, () => {

            });
        }

        /// <summary>
        /// 直玩订阅查询
        /// </summary>
        /// <param name="scene">订阅的场景ID。1：离线收益场景​；2：体力恢复场景​；3：重要事件掉落​</param>
        /// <param name="action">isSuccess：用户的订阅结果（true为成功，false为订阅失败）；message：回调信息 ​</param>
        public void CheckFeedSubscribeStatus(int scene, Action<bool, string> action = null)
        {
            var param = new JsonData();
            param["type"] = "play";// play=直玩（该场景必传scene和contentIDs）
            param["scene"] = scene;

            FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("直玩订阅查询业务-NebulaSDKBridge.CheckFeedSubscribeStatus").AddExtra("param", param.ToJson()).Report();

            TT.CheckFeedSubscribeStatus(param, (res) => {
                bool success = (bool)res["success"];
                string errMsg = (string)res["errMsg"];

                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("直玩订阅查询业务-完成回调").AddExtra("success", $"{success}").AddExtra("errMsg", errMsg).Report();

                action?.Invoke(success, errMsg);
            }, (errNo, errMsg) => {
                bool success = false;

                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("直玩订阅查询业务-错误回调").AddExtra("success", $"{success}").AddExtra("errNo", $"{errNo}").AddExtra("errMsg", errMsg).Report();

                action?.Invoke(success, errMsg);
            }, () => {

            });
        }

        /// <summary>
        /// 自定义启动场景数据上报接口
        /// 向平台侧传达游戏场景加载完成、达到可交互状态的时机
        /// </summary>
        /// <param name="param">sceneId：场景ID，int类型(直玩场景传7001)；costTime：场景耗时，单位 ms long类型</param>
        /// <param name="action">isSuccess：上报结果（true为成功，false为失败）；message：回调信息 ​ ​</param>
        public void ReportScene(JsonData param, Action<bool, string> action = null)
        {
            FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("数据上报业务").AddExtra("param", $"{param.ToJson()}").Report();
            TT.ReportScene(param, (res) =>
            {
                action?.Invoke(true, "上报成功");
            }, (code, msg) =>
            {
                action?.Invoke(false, msg);
            }, () =>
            {
               
            });
        }

        /// <summary>
        /// 设置调试模式
        /// </summary>
        /// <param name="isDebug">
        /// 打开调试模式：true
        /// 关闭调试模式：falese
        /// </param>
        public void NebulaDebug(bool isDebug)
        {
            this.isDebug = isDebug;
        }

        /// <summary>
        /// 抖音支付
        /// 只支持Android
        /// </summary>
        /// <param name="zoneId"></param>
        /// <param name="goodsPrice"></param>
        /// <param name="orderCreate"></param>
        private void DirectPay(string zoneId, int goodsPrice, FYResponseModel.BaseResponse<FYResponseModel.OrderCreate> orderCreate)
        {
            Debug.Log($"###抖音支付");
            
            if (TTSDK.CanIUse.RequestGamePayment)
            {
                ExtraInfo extraInfo = new ExtraInfo();
                extraInfo.zone_id = zoneId;
                extraInfo.pay_time = $"{System.DateTimeOffset.UtcNow.ToUnixTimeSeconds()}";
                extraInfo.amount = goodsPrice;
                string extraJson = JsonUtility.ToJson(extraInfo);

                Dictionary<string, object> orderInfoParams = new Dictionary<string, object>();
                orderInfoParams["mode"] = "game";
                orderInfoParams["env"] = "0";
                orderInfoParams["currencyType"] = "CNY"; // 固定值: CNY。币种
                orderInfoParams["platform"] = "android";
                orderInfoParams["buyQuantity"] = goodsPrice / 100;
                orderInfoParams["customId"] = orderCreate.data.order_id;
                orderInfoParams["extraInfo"] = extraJson;

                FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-拉起抖音支付").AddExtra("buyQuantity", $"{orderInfoParams["buyQuantity"]}").AddExtra("customId", $"{orderInfoParams["customId"]}").AddExtra("extraInfo", $"{orderInfoParams["extraInfo"]}").Report();

                TT.RequestGamePayment(orderInfoParams,
                    () =>
                    {
                        Debug.Log($"###抖音支付Success");

                        var info = new NebulaSDKPayInfo();
                        info.status = "success";
                        info.code = 0;
                        info.msg = "支付成功";
                        var data = new NebulaSDKPayData();
                        data.payType = 1;
                        data.orderNo = orderCreate.data.order_id;
                        info.data = data;

                        FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-抖音支付成功-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                        NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
                    },
                    (errCode, errMsg) =>
                    {
                        Debug.Log($"###抖音支付Failure>>errCode:{errCode}>>errMsg:{errMsg}");

                        var info = new NebulaSDKPayInfo();
                        info.status = "error";
                        info.code = NebulaSDKCode.Code30001;
                        info.msg = $"支付失败:{errMsg}";

                        FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-抖音支付失败-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).AddExtra("errCode", $"{errCode}").AddExtra("errMsg", errMsg).Report();

                        NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
                    }
                );
            }
            else
            {
                Debug.Log($"###Pay Not CanIUse");

                var info = new NebulaSDKPayInfo();
                info.status = "error";
                info.code = NebulaSDKCode.Code30001;
                info.msg = "支付不能用";

                FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-抖音支付不能用-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
            }
        }

        /// <summary>
        /// 发起抖音钻石支付
        /// 只支持iOS
        /// </summary>
        /// <param name="zoneId"></param>
        /// <param name="goodsPrice"></param>
        /// <param name="orderCreate"></param>
        private void DiamondPay(string zoneId, int goodsPrice, FYResponseModel.BaseResponse<FYResponseModel.OrderCreate> orderCreate)
        {
            Debug.Log($"###抖音钻石支付");
            ExtraInfo extraInfo = new ExtraInfo();
            extraInfo.zone_id = zoneId;
            extraInfo.pay_time = $"{System.DateTimeOffset.UtcNow.ToUnixTimeSeconds()}";
            extraInfo.amount = goodsPrice;
            string extraJson = JsonUtility.ToJson(extraInfo);

            JsonData orderInfoParams = new JsonData();
            orderInfoParams["buyQuantity"] = goodsPrice / 100;
            orderInfoParams["customId"] = orderCreate.data.order_id;
            orderInfoParams["currencyType"] = "DIAMOND";
            orderInfoParams["zoneId"] = zoneId;
            orderInfoParams["extraInfo"] = extraJson;

            FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-拉起钻石支付").AddExtra("orderInfoParams", orderInfoParams.ToJson()).Report();

            TT.OpenAwemeCustomerService(orderInfoParams, () => {
                Debug.Log($"###抖音钻石支付Success");

                var info = new NebulaSDKPayInfo();
                info.status = "success";
                info.code = 0;
                info.msg = "支付成功";
                var data = new NebulaSDKPayData();
                data.payType = 1;
                data.orderNo = orderCreate.data.order_id;
                info.data = data;

                FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-钻石支付成功-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
            }, (errCode, errMsg) => {
                Debug.Log($"###抖音钻石支付Failure>>errCode:{errCode}>>errMsg:{errMsg}");

                var info = new NebulaSDKPayInfo();
                info.status = "error";
                info.code = NebulaSDKCode.Code30001;
                info.msg = $"支付失败:{errMsg}";

                FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-钻石支付失败-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
            });
        }

        /// <summary>
        /// 发起抖音IM客服支付
        /// 只支持iOS
        /// </summary>
        /// <param name="zoneId"></param>
        /// <param name="goodsPrice"></param>
        /// <param name="orderCreate"></param>
        private void ServicePay(string zoneId, int goodsPrice, FYResponseModel.BaseResponse<FYResponseModel.OrderCreate> orderCreate)
        {
            Debug.Log($"###抖音IM客服支付");
            ExtraInfo extraInfo = new ExtraInfo();
            extraInfo.zone_id = zoneId;
            extraInfo.pay_time = $"{System.DateTimeOffset.UtcNow.ToUnixTimeSeconds()}";
            extraInfo.amount = goodsPrice;
            string extraJson = JsonUtility.ToJson(extraInfo);

            JsonData orderInfoParams = new JsonData();
            orderInfoParams["buyQuantity"] = goodsPrice / 100;
            orderInfoParams["customId"] = orderCreate.data.order_id;
            orderInfoParams["currencyType"] = "CNY";
            orderInfoParams["zoneId"] = zoneId;
            orderInfoParams["extraInfo"] = extraJson;

            FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-拉起抖音IM客服支付").AddExtra("orderInfoParams", orderInfoParams.ToJson()).Report();

            TT.OpenAwemeCustomerService(orderInfoParams, () => {
                Debug.Log($"###抖音IM客服支付Success");

                var info = new NebulaSDKPayInfo();
                info.status = "success";
                info.code = 0;
                info.msg = "支付成功";
                var data = new NebulaSDKPayData();
                data.payType = 1;
                data.orderNo = orderCreate.data.order_id;
                info.data = data;

                FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-抖音IM客服支付成功-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
            }, (errCode, errMsg) => {
                Debug.Log($"###抖音IM客服支付Failure>>errCode:{errCode}>>errMsg:{errMsg}");

                var info = new NebulaSDKPayInfo();
                info.status = "error";
                info.code = NebulaSDKCode.Code30001;
                info.msg = $"支付失败:{errMsg}";

                FYLogService.Create().SetEventId(FYLogService.PAY).SetDesc("支付业务-抖音IM客服支付失败-回调NebulaSDKBridgeCallback.PayCallback").AddExtra("info", JsonUtility.ToJson(info)).Report();

                NebulaSDKBridgeCallback.PayCallback?.Invoke(info);
            });
        }


        [System.Serializable]
        private class ExtraInfo
        {
            public string zone_id;
            public string pay_time;
            public int amount;
        }

        /// <summary>
        ///  刚打开游戏时获取启动参数
        ///  给游戏回调OnShow
        /// </summary>
        private void GetLaunchOptions()
        {
            var launchOptions = TT.GetLaunchOptionsSync();

            foreach (KeyValuePair<string, string> pair in launchOptions.Extra)
            {
                Debug.Log("###launchOptions.Extra>>Key: " + pair.Key + ", Value: " + pair.Value);
            }

            foreach (KeyValuePair<string, string> pair in launchOptions.Query)
            {
                Debug.Log("###launchOptions.Query>>Key: " + pair.Key + ", Value: " + pair.Value);
            }

            var onShowData = new NebulaSDKOnShowData();

            // 判断是否是直出场景
            string scene = launchOptions.Scene;
            if (scene != null && scene.Contains("3041"))
            {
                onShowData.isFeedGameScene = true;
            }
            else
            {
                onShowData.isFeedGameScene = false;
            }

            string feedGameScene = "";
            string feedGameExtra = "";
            if (launchOptions.Query != null)
            {
                // 获取直出场景值
                if (launchOptions.Query.ContainsKey("feed_game_scene"))
                {
                    feedGameScene = launchOptions.Query["feed_game_scene"];
                    onShowData.feedGameScene = int.Parse(feedGameScene);
                    Debug.Log($"###feedGameScene:{onShowData.feedGameScene}");
                }
                // 获取直出开发者自定义字段
                if (launchOptions.Query.ContainsKey("feed_game_extra"))
                {
                    feedGameExtra = launchOptions.Query["feed_game_extra"];
                    onShowData.feedGameExtra = feedGameExtra;
                    Debug.Log($"###feedGameExtra:{onShowData.feedGameExtra}");
                }
            }

            var launchFrom = "";
            var location = "";

            if (launchOptions.Extra != null)
            {
                if (launchOptions.Extra.ContainsKey("launchFrom"))
                {
                    launchFrom = launchOptions.Extra["launchFrom"];
                    Debug.Log($"###launchFrom:{launchFrom}");
                }

                if (launchOptions.Extra.ContainsKey("launch_from"))
                {
                    launchFrom = launchOptions.Extra["launch_from"];
                    Debug.Log($"###launch_from:{launchFrom}");
                }

                if (launchOptions.Extra.ContainsKey("location"))
                {
                    location = launchOptions.Extra["location"];
                    Debug.Log($"###location:{location}");
                }
            }

            // 判断是否从侧边栏跳转
            if (launchFrom == "homepage" && location == "sidebar_card")
            {
                onShowData.fromSideBar = true;
            }
            else
            {
                onShowData.fromSideBar = false;
            }

            FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("第三方业务-获取launchOptions参数").AddExtra("scene", scene).AddExtra("feed_game_scene", feedGameScene).AddExtra("feed_game_extra", feedGameExtra).AddExtra("location", location).AddExtra("launch_from", launchFrom).Report();

            // 检查是否支持侧边栏
            TT.CheckScene(TTSideBar.SceneEnum.SideBar, supportSideBar =>
            {
                Debug.Log("###检查是否支持侧边栏成功:" + supportSideBar);

                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("第三方业务-检查是否支持侧边栏成功").AddExtra("supportSideBar", $"{supportSideBar}").Report();

                this.supportSideBar = supportSideBar;

                onShowData.supportSideBar = supportSideBar;

                CallbackOnShow(onShowData);
            }, () =>
            {
                Debug.Log("###CheckScene Complete");
            }, (errCode, errMsg) =>
            {
                Debug.Log($"###检查是否支持侧边栏失败>>errCode:{errCode}>>errMsg:{errMsg}");

                FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("第三方业务-检查是否支持侧边栏失败").Report();

                onShowData.supportSideBar = false;

                CallbackOnShow(onShowData);
            });
        }

        /// <summary>
        /// 监听抖音OnShow
        /// </summary>
        /// <param name="param"></param>
        private void OnShow(Dictionary<string, object> param)
        {
            foreach (KeyValuePair<string, object> pair in param)
            {
                Debug.Log("###OnShow>>Key: " + pair.Key + ", Value: " + pair.Value);
            }

            var onShowData = new NebulaSDKOnShowData();

            var launchFrom = "";
            var location = "";

            if (param != null)
            {
                if (param.ContainsKey("launchFrom"))
                {
                    launchFrom = (string)param["launchFrom"];
                    Debug.Log($"###launchFrom:{launchFrom}");
                }

                if (param.ContainsKey("launch_from"))
                {
                    launchFrom = (string)param["launch_from"];
                    Debug.Log($"###launch_from:{launchFrom}");
                }

                if (param.ContainsKey("location"))
                {
                    location = (string)param["location"];
                    Debug.Log($"###location:{location}");
                }
            }

            // 判断是否从侧边栏跳转
            if (launchFrom == "homepage" && location == "sidebar_card")
            {
                onShowData.fromSideBar = true;
            }
            else
            {
                onShowData.fromSideBar = false;
            }

            FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("第三方业务-监听OnShow").AddExtra("location", location).AddExtra("launchFrom", launchFrom).Report();
            onShowData.supportSideBar = this.supportSideBar;

            CallbackOnShow(onShowData);
        }

        /// <summary>
        /// 给游戏回调OnShow回调
        /// </summary>
        /// <param name="supportSideBar"></param>
        /// <param name="fromSideBar"></param>
        private void CallbackOnShow(NebulaSDKOnShowData onShowData)
        {
            var onShowInfo = new NebulaSDKOnShowInfo();
            onShowInfo.status = "success";
            onShowInfo.code = 0;
            onShowInfo.msg = "OnShow回调成功";
            onShowInfo.data = onShowData;

            FYLogService.Create().SetEventId(FYLogService.THIRD).SetDesc("第三方业务-回调NebulaSDKBridgeCallback.OnShow").AddExtra("onShowInfo", JsonUtility.ToJson(onShowInfo)).Report();

            NebulaSDKBridgeCallback.OnShow?.Invoke(onShowInfo);
        }

        
    }
}

    
