﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using System;
using Nebula;
using StarkSDKSpace;
using TTSDK.UNBridgeLib.LitJson​;


public class SDKManager 
{
    public static string GameVersion = "1.0.0";

    public bool IsInitFinish { get; private set; }

    public bool IsLoginSuccess { get; private set; }

    public string NebulaEnv { get; private set; }

    public string GoodsId { get; private set; }

    public string GoodsName { get; private set; }

    public string GameOrderId { get; private set; }

    public int PayAmount { get; private set; }

    public string AppId { get; private set; }

    public string ChannelId { get; private set; }

    public string ChannelTemplateId { get; private set; }

    public string ChannelTemplateCode { get; private set; }

    public string PassportCode { get; private set; }

    public string PaymentCode { get; private set; }

    public string DeviceId { get; private set; }

    public string ADTrackingData { get; private set; }

    public string SDKVersion { get; private set; }

    public string Host { get; private set; }

    public NebulaSDKLoginInfo LoginSuccessInfo { get; private set; }

    public NebulaSDKOnShowInfo OnShowInfo { get; private set; }

    public static string PlayerID = "user001";

    public static string PlayerName = "玩家001";

    public static string PlayerLevel = "1";

    public static string ServerID = "1";

    public static string ZoneID = "1";

    public static string ServerName = "区服001";

    public static string RegTime = "1724221316";

    public static string[] Goods = new string[] { "com.feiyu.mjt.libao1"};

    public static string[] ADS = new string[] { "reward_1" };

    private static SDKManager instance;

    public static SDKManager Instance
    {
        get
        {
            if (instance == null)
            {
                instance = new SDKManager();
            }
            return instance;
        }
    }

    private SDKManager()
    {

        var json = Resources.Load<TextAsset>("fy-config");
        Debug.Log($"###fy-config>>{json}");
        var config = JsonUtility.FromJson<Nebula.NebulaSDKConfig>(json.text);
        // 星云开放平台AppID 
        AppId = config.nbAppId;
        // 星云开放平台渠道ID
        ChannelId = config.nbChannelId;
        
        // 星云开放平台AppID 
        var appId = config.nbAppId;
        // 星云开放平台渠道ID
        var channelId = config.nbChannelId;
        // 星云开放平台渠道模板标识
        var channelTemplateCode = config.nbChannelTemplateCode;
        // 星云开放平台通行证标识
        var passportCode = config.passportCode;
        // 星云开放平台支付标识
        var paymentCode = config.paymentCode;

        SDKVersion = NebulaSDKBridge.SDKVersion;
        ContainerEnv env = StarkSDK.s_ContainerEnv;
        HostEnum host = env.m_HostEnum;
        Host = $"{host}";

        Nebula.NebulaSDKBridgeCallback.InitCallback = (initInfo) =>
        {
            LogManager.Instance.Add($"###InitCallback>>status:{initInfo.status}>>code:{initInfo.code}>>msg:{initInfo.msg}>>nbAppId:{initInfo.data.nbAppId}>>nbChannelId:{initInfo.data.nbChannelId}");

            if ("success".Equals(initInfo.status))
            {
                // 初始化成功
                LogManager.Instance.Add("###初始化成功");
                IsInitFinish = true;
            }
            else
            {
                // 初始化失败
                LogManager.Instance.Add("###初始化失败");
                IsInitFinish = false;
            }
        };

        Nebula.NebulaSDKBridgeCallback.LoginCallback = (loginInfo) =>
        {
            LogManager.Instance.Add("###LoginCallback>>status:" + loginInfo.status + ">>code:" + loginInfo.code + ">>msg:" + loginInfo.msg + ">>openId:" + loginInfo.data.openId + ">>token:" + loginInfo.data.token + ">>nbOpenId:" + loginInfo.data.nbOpenId);

            if ("success".Equals(loginInfo.status))
            {
                // 登录成功
                LogManager.Instance.Add("###登录成功");

                this.LoginSuccessInfo = loginInfo;
                this.IsLoginSuccess = true;
            }
            else
            {
                // 登录失败
                LogManager.Instance.Add($"###登录失败");
                this.IsLoginSuccess = false;
            }
        };

        Nebula.NebulaSDKBridgeCallback.PayCallback = (payInfo) =>
        {
            LogManager.Instance.Add("###PayCallback>>status:" + payInfo.status + ">>code:" + payInfo.code + ">>msg:" + payInfo.msg + ">>orderNo:" + payInfo.data.orderNo + ">>payType:" + payInfo.
                        data.payType);

            if ("success".Equals(payInfo.status))
            {
                // 支付成功
                LogManager.Instance.Add("###支付成功");
            }
            else
            {
                // 支付失败
                LogManager.Instance.Add("###支付失败");
            }
        };

        Nebula.NebulaSDKBridgeCallback.OnShow = (info) =>
        {
            LogManager.Instance.Add("###OnShow>>status:" + info.status + ">>code:" + info.code + ">>msg:" + info.msg + ">>supportSideBar:" + info.data.supportSideBar + ">>fromSideBar:" + info.
                        data.fromSideBar + ">>isFeedGameScene:" + info.data.isFeedGameScene + ">>feedGameScene:" + info.data.feedGameScene + ">>feedGameExtra:" + info.data.feedGameExtra);

            this.OnShowInfo = info;
        };
    }

    public void DoInit()
    {
        LogManager.Instance.Add("###Nebula.NebulaSDKBridge.Instance.Init");

        Nebula.NebulaSDKBridge.Instance.Init(GameVersion);
    }

    public void DoLogin()
    {
        LogManager.Instance.Add("###Nebula.NebulaSDKBridge.Instance.Login");

        Nebula.NebulaSDKBridge.Instance.Login();
    }

    public void DoSwitchAccount()
    {
        
    }

    public void NavigateToSideBar()
    {
        LogManager.Instance.Add("###Nebula.NebulaSDKBridge.Instance.NavigateToSideBar");

        Nebula.NebulaSDKBridge.Instance.NavigateToSideBar(() => {
            LogManager.Instance.Add("###打开侧边栏成功");
        }, (errCode, errMsg) => {
            LogManager.Instance.Add("###打开侧边栏失败");
        });
    }

    public void DoLogout()
    {
        LogManager.Instance.Add("###退出到登录首页");

        IsLoginSuccess = false;

        SceneManager.LoadScene(0);
    }

    public void DoPay()
    {
        GoodsId = Goods[0];
        GoodsName = "100钻石";
        long gameOrderId = ((DateTimeOffset)DateTime.UtcNow).ToUnixTimeSeconds();
        GameOrderId = $"{gameOrderId}";
        PayAmount = 100;

        LogManager.Instance.Add($"###Nebula.NebulaSDKBridge.Instance.Pay>>GoodsId:{GoodsId}>>GoodsName:{GoodsName}>>GameOrderId:{GameOrderId}>>PayAmount:{PayAmount}");

        Nebula.NebulaSDKBridge.Instance.Pay(PayAmount, GameOrderId, GoodsId, GoodsName, PlayerID, PlayerName, PlayerLevel, "1", ServerID, "1", 0, "", "");
    }

    public void CreateShortcut()
    {
        LogManager.Instance.Add("###Nebula.NebulaSDKBridge.Instance.CreateShortcut");

        Nebula.NebulaSDKBridge.Instance.CreateShortcut((isSuccess) => {
            LogManager.Instance.Add($"###创建桌面快捷方式​>>isSuccess:{isSuccess}");
        });
    }

    public void QueryLocalProducts()
    {

    }

    public void AntiAddiction()
    {

    }

    public void CheckAuth()
    {
       
    }

    public void ServiceXiao6()
    {
        LogManager.Instance.Add("###Nebula.NebulaSDKBridge.Instance.OpenCustomerServiceConversation>>打开小6客服");
        Nebula.NebulaSDKBridge.Instance.OpenCustomerServiceConversation(1, (success) => {
            LogManager.Instance.Add($"###打开小6客服>>success:{success}");
        });
    }

    public void ServiceIM()
    {
        LogManager.Instance.Add("###Nebula.NebulaSDKBridge.Instance.OpenCustomerServiceConversation>>打开抖音IM客服");
        Nebula.NebulaSDKBridge.Instance.OpenCustomerServiceConversation(2, (success) => {
            LogManager.Instance.Add($"###打开抖音IM客服>>success:{success}");
        });
    }

    public void DoADTrackingReport(string method)
    {
        
    }

    public void LoadReward()
    {


    }

    public void ShowReward()
    {
        
    }

    public void RequestFeedSubscribe()
    {
        LogManager.Instance.Add("###Nebula.NebulaSDKBridge.Instance.RequestFeedSubscribe");
        string[] contentIDs = new string[] { "aaa", "bbb", "xxx" };
        Nebula.NebulaSDKBridge.Instance.RequestFeedSubscribe(1, contentIDs, (isSuccess, message) => {
            LogManager.Instance.Add($"###订阅直出结果>>isSuccess:{isSuccess}>>message:{message}");
        });
    }

    public void ReportScene()
    {
        LogManager.Instance.Add("###Nebula.NebulaSDKBridge.Instance.ReportScene");
        var param = new JsonData();
        param["sceneId"] = 7001;
        param["costTime"] = 100;
        Nebula.NebulaSDKBridge.Instance.ReportScene(param, (isSuccess, message) => {
            LogManager.Instance.Add($"###上报结果>>isSuccess:{isSuccess}>>message:{message}");
        });
    }
}
